#include "StdAfx.h"
#include "textparser.h"

CTextParser::CTextParser(void)
{
	m_Created = OT_NONE;
}

CTextParser::~CTextParser(void)
{
	if (m_Created)
	{
		Destroy();
	}
}

OpenType CTextParser::Create(const char *fname, bool Write, bool Overwrite)
{
	if (m_Created)
	{
		//    ,      
		Destroy();
	}

	m_Created = OT_NONE;
	if (!Write)
	{
		m_fp.open(fname, ios::in);

		if (m_fp.is_open())
			m_Created = OT_READ;
	}
	else
	{
		if (Overwrite)
		{
			m_fp.open(fname, ios::out | ios::trunc);
		}
		else
		{
			m_fp.open(fname, ios::out | ios::app);
		}

		if (m_fp.is_open())
		{
			m_Created = OT_WRITE;
		}
	}
	m_CurLine = 0;
	if (!m_Created)
	{
		m_strError = "    ";
		m_strError += fname;
		m_strError += "\n";
		m_ErrNumber = ERR_OPENFILE;
	}
	return m_Created;
}

void CTextParser::Destroy()
{
	m_fp.close();
	m_Created = OT_NONE;
	m_fp.clear();
	m_strError.erase();
}

bool CTextParser::GetNextValue(StringData *info)
{
	string CurStr;
	string strName;								//  
	string strValue;							//  

	bool res;
	if (m_Created != OT_READ)
	{
		m_strError = "     \n";
		m_ErrNumber = ERR_NOTCREATE;
		return false;
	}

	do 
	{
		res = !!std::getline(m_fp, CurStr);
		TrimString(&CurStr);
		m_CurLine++;
	} while(res && (CurStr.empty() || CurStr[0] == ';'));				//   ,   . ';'    - 
	
	if (!res)
	{
		//     
		m_ErrNumber = ERR_ENDOFFILE;
		m_strError = " \n";
		return false;
	}

	//    2:   
	if (!SeparateString(CurStr, &strName, &strValue))
	{
		//   
		info->Type = ST_NAMEONLY;
		TrimString(&CurStr);
		info->Name = CurStr;
	}
	else
	{
		TrimString(&strName);
		TrimString(&strValue);
		info->Name = strName;

		//    
		GetType (strValue, info);
	} // (!SeparateString(CurStr, strName, strValue))

	//logFile.WriteString("%s\n", strValue.c_str());
	return (info->Type != ST_ERROR);
}

//       \t
void CTextParser::TrimString(string *str)
{
	size_t bpos;								//    
	size_t epos;								//    

	bpos = str->find_first_not_of(" \t\n\r");
	if (bpos != string::npos)
	{
		str->erase(0, bpos);
	}

	epos = str->find_last_not_of(" \t\n\r");
	if (epos != string::npos)
	{
		str->erase(epos + 1, string::npos);
	}
}

bool CTextParser::SeparateString(string strIn, string *strName, string *strVal)
{
	bool res;
	size_t pos = strIn.find('=');
	if (pos == string::npos)
	{
		res = false;
	}
	else
	{
		*strName = strIn.substr(0, pos);
		*strVal = strIn.substr(pos + 1);
		res = true;
	}

	return res;
}

void CTextParser::GetType (string str, StringData *info)
{
//	double fVal;
//	int iVal;
	size_t len;

	if (str[0] == '\"')
	{
		info->Type = ST_STRING;
		len = str.length();
		
		//      (   ? :))
		if (str[len - 1] == '\"')
		{
			info->strVal = str.substr (1, len - 2);
		}
		else
		{
			info->strVal = str.substr (1);
		}
	}	// if (str[0] == '\"')
	else
	{
		//    ?
		if (IsInt (str))
		{
			// 
			info->intVal = atoi(str.c_str());
			info->Type = ST_INT;		
		}
		else
		{
			//   .   float
			if (IsFloat(str))
			{
				// , float
				info->fVal = atof (str.c_str());
				info->Type = ST_FLOAT;
			}
			else if (!CustomType(str, info))
			{
				//    - 
				info->Type = ST_ERROR;
				//  -   
				info->strVal = str;
				m_strError = "   \n";
				m_ErrNumber = ERR_TYPE;
			}
		}	// if (str.find_last_not_of('0') == string::npos) else
	}	// if (str[0] == '\"') else
}

bool CTextParser::IsFloat(string str)
{
	bool res = false;
	size_t pos;

	// ,        
	if (str.find_last_not_of("0123456789.-") == string::npos)
	{
		// ,       "-"    
		pos = str.find('.');
		if ((pos != string::npos && str.find('.', pos + 1) == string::npos) &&
			(str.find('.', 1) != string::npos))
		{
			res = true;
		}

	}

	return res;
}

void CTextParser::GetTextError (string *str)
{
	*str = m_strError;
}

int CTextParser::GetCurLine()
{
	return m_CurLine;
}

bool CTextParser::IsInt(string str)
{
	return str.find_first_not_of("0123456789-") == string::npos;
}

bool CTextParser::WriteBuf (string str)
{
	//using namespace std;
	if (m_Created != OT_WRITE)
	{
		m_strError = "     \n";
		m_ErrNumber = ERR_NOTCREATE;
		return false;
	}

	m_fp << str << std::endl;
	return true;
}

bool CTextParser::Write (string name, string val)
{
	string buf = name + " = " + val;

	return WriteBuf (buf);
}

bool CTextParser::Write (string name, int val)
{
	char buf[256];
	itoa (val, buf, 10);
	return Write (name, buf);
}

bool CTextParser::Write (string name, float val)
{
	char buf[256];
	sprintf(buf, "%s = %f", name.c_str(), val);
	return WriteBuf (buf);
}

bool CTextParser::Write (string name, double val)
{
	char buf[256];
	sprintf(buf, "%s = %f", name.c_str(), val);
	return WriteBuf (buf);
}

bool CTextParser::Write (StringData *info)
{
	switch(info->Type) 
	{
	case ST_STRING:
		return Write (info->Name, info->strVal);
	case ST_INT:
		return Write(info->Name, info->intVal);
	case ST_FLOAT:
		return Write(info->Name, info->fVal);
	default:
		return false;
	}
}

bool CTextParser::WriteComment(string str)
{
	return WriteBuf ("; " + str);
}